@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm, aCaB
@@ -34,10 +35,9 @@
 #include <time.h>
 #include <signal.h>
 
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include "libclamav/crypto.h"
+#include "clamav.h"
 
+#include "libclamav/clamav.h"
 #include "shared/output.h"
 #include "shared/misc.h"
 #include "shared/optparser.h"
@@ -69,10 +69,6 @@ int main(int argc, char **argv)
 	struct sigaction sigact;
 #endif
 
-#if defined(_WIN32)
-    cl_initialize_crypto();
-#endif
-
     if((opts = optparse(NULL, argc, argv, 1, OPT_CLAMDSCAN, OPT_CLAMSCAN, NULL)) == NULL) {
 	mprintf("!Can't parse command line options\n");
 	return 2;
@@ -177,6 +173,7 @@ int main(int argc, char **argv)
 
     logg_close();
     optfree(opts);
+    cl_cleanup_crypto();
     exit(ret);
 }
 
@@ -187,8 +184,8 @@ void help(void)
 
     mprintf("\n");
     mprintf("                       ClamAV Daemon Client %s\n", get_version());
-    printf("           By The ClamAV Team: http://www.clamav.net/team\n");
-    printf("           (C) 2007-2009 Sourcefire, Inc.\n\n");
+    printf("           By The ClamAV Team: http://www.clamav.net/about.html#credits\n");
+    printf("           (C) 2007-2015 Cisco Systems, Inc.\n\n");
 
     mprintf("    --help              -h             Show help\n");
     mprintf("    --version           -V             Print version number and exit\n");
