@@ -1,4 +1,5 @@
 /*
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2009 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
@@ -37,6 +38,14 @@
 # endif
 #endif
 
+#ifdef HAVE_SYSTEMD
+# include <systemd/sd-daemon.h>
+#else
+# define sd_listen_fds(u) 0
+# define SD_LISTEN_FDS_START 3
+# define sd_is_socket(f, a, s, l) 1
+#endif
+
 #include <limits.h>
 
 #ifndef PATH_MAX
@@ -51,7 +60,6 @@ int filecopy(const char *src, const char *dest);
 int daemonize(void);
 const char *get_version(void);
 int match_regex(const char *filename, const char *pattern);
-int cfg_tcpsock(const struct optstruct *opts, struct sockaddr_in *server, in_addr_t defaultbind);
 int cli_is_abspath(const char *path);
 unsigned int countlines(const char *filename);
 
