@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2007-2009 Sourcefire, Inc.
+ *  Copyright (C) 2015 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Tomasz Kojm
  *
@@ -25,7 +26,12 @@
 #include "clamav-config.h"
 #endif
 
+#if HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 
 #ifdef __GNUC__
 int mdprintf(int desc, const char *str, ...) __attribute__((format(printf, 2,3)));
@@ -55,6 +61,6 @@ void mprintf(const char *str, ...) __attribute__((format(printf, 1, 2)));
 void mprintf(const char *str, ...);
 #endif
 
-extern short int mprintf_disabled, mprintf_verbose, mprintf_quiet, mprintf_nowarn, mprintf_stdout, mprintf_send_timeout;
+extern short int mprintf_disabled, mprintf_verbose, mprintf_quiet, mprintf_nowarn, mprintf_stdout, mprintf_send_timeout, mprintf_progress;
 
 #endif
